local json = require("json")
local utils = require("utils")
local const = require("const")
local Set = require("set")
local resty_string = require("resty.string")
local hmac = require("resty.hmac")
local resty_random = require("resty.random")
local pl_pretty = require("pl.pretty")

local request_method = ngx.var.request_method

utils.check_login()
_, login_token = utils.check_permission("GatewayManage")

local function del_standard_tsl(pk)
    _, msg = utils.set_data(const.data_key.tsl.standard_tsl_prefix..pk, "")
    
    if (msg == nil) then
        msg = "{}"
    end

    return msg
end

if "POST" == request_method then
    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
    if(nil == args_json) then
        utils.error_exit(400, json.encode("request is invalid"))
    end

    args = json.decode(args_json)
    if (nil == args) then
        utils.error_exit(400)
    end

    local request = args["request"]
    if (request == nil) then
        utils.error_exit(400, json.encode("request is invalid"))
    end

    local apiVer = request["apiVer"] 
    if (apiVer ~= "0.6") then
        utils.error_exit(400, json.encode("apiVer is invalid"))
    end
    
    local params = args["params"]
    if(nil == params) then
        utils.error_exit(400, json.encode("params should not be null"))
    end
   
    local productKeyList = {}
    local productKey = params["productKey"]
  
    if (productKey == nil) then
        productKeyList = json.decode(utils.get_tsl_key_list()) 
    else
        for _, pk in ipairs(productKey) do 
            utils.check_pk(pk)
            table.insert(productKeyList, const.data_key.tsl.standard_tsl_prefix..pk) 
        end
    end

    for _, pk in ipairs(productKeyList) do
        del_standard_tsl(pk)
    end

    utils.error_exit(200, json.encode("")) 
end
utils.error_exit(400, json.encode("unsupported request method"))

------ EOF ------
