local json = require("json")
local utils = require("utils")
local const = require("const")
local Set = require("set")
local resty_string = require("resty.string")
local hmac = require("resty.hmac")
local resty_random = require("resty.random")
local pl_pretty = require("pl.pretty")

local request_method = ngx.var.request_method

utils.check_login()
_, login_token = utils.check_permission("TSLManage")

local function del_tsl(pk)
    _, msg = utils.set_data(pk, "")
    
    if (msg == nil) then
        return false
    end

    return true
end

if "POST" == request_method then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_body_data()
    local standard_tsl_pk_list = {}
    local extend_tsl_pk_list = {}
    local productKey = params["productKey"]

    if (productKey == nil or type(productKey) ~= "table") then
        utils.error_exit(400, "product key must be array")
    end

    if (#productKey == 0) then
        standard_tsl_pk_list = json.decode(utils.get_standard_tsl_key_list()) 
        extend_tsl_pk_list = json.decode(utils.get_extend_tsl_key_list()) 
    else
        for _, pk in pairs(productKey) do 
            utils.check_pk(pk)
            table.insert(standard_tsl_pk_list, const.data_key.tsl.standard_tsl_prefix..pk) 
            table.insert(extend_tsl_pk_list, const.data_key.tsl.extend_tsl_prefix..pk) 
        end
    end

    local ok, pk_gw = utils.get_data(const.data_key.gwconfig.productkey)
    if ok and pk_gw ~= nil then 
        pk_gw = const.data_key.tsl.standard_tsl_prefix..pk_gw
    end

    -- we should not delete the gateway tsl.
    local ret = {}
    for _, pk in pairs(standard_tsl_pk_list) do
        if pk ~= pk_gw then
            del_tsl(pk)
        end
    end

    for _, pk in pairs(extend_tsl_pk_list) do
        del_tsl(pk)
    end

    utils.error_exit(200, "{}") 
end
utils.error_exit(400, "unsupported request method")

------ EOF ------
