local json = require("json")
local utils = require("utils")
local const = require("const")
local request_method = ngx.var.request_method

utils.check_login()
_, login_token = utils.check_permission("StreamCompute")

local function check_sort_method(method)
    if method == nil or method == "TimeDesc" then
        return "created_time", "desc"
    elseif method == "TimeAsc" then
        return "created_time", "asc"
    elseif method == "NameDesc" then
        return "name", "desc"
    elseif method == "NameAsc" then
        return "name", "asc"
    elseif method == "StatusDesc" then
        return "status", "desc"
    elseif method == "StatusAsc" then
        return "status", "asc"
    else
        return "created_time", "desc"
    end
end

if "POST" == request_method then
    local utils_header = require("utils_header")
    local params = utils_header.check_public_body_data()
    local pageSize = params["PageSize"]
    local currentPage = params["CurrentPage"]
    local sortMethod = params["SortMethod"]
    
    utils.check_current_page(currentPage)
    pageSize = utils.check_page_size_allow_zero(pageSize)
    
    local cmd = {}
    cmd["pageSize"] = pageSize
    cmd["curPage"] = currentPage
    cmd["sortBy"], cmd["sortOrder"] = check_sort_method(sortMethod)

    local ok, msg_raw, err = utils.dbus_call_edge(const.stream_compute.wkn,
                                        const.stream_compute.obj,
                                        const.stream_compute.inf,
                                        const.stream_compute.method.getJobList,
                                        json.encode(cmd))

    local msg_ret = {}
    if (ok and msg_raw) then
        local ok, msg = pcall(json.decode, msg_raw[1])
        if (ok and 0 == msg["code"]) then
            msg_ret["TotalNum"] = msg["data"]["totalCount"]
            msg_ret["PageSize"] =  msg["data"]["pageSize"]
            msg_ret["CurrentPage"] = msg["data"]["curPage"]
          
	    local tmp_tab = {}
            for _, v in pairs(msg["data"]["jobList"]) do
                local item = {}
                item["TaskNO"] = v["id"] 
                item["Status"] = v["running"] 
                item["Name"] = v["name"]
                item["Desc"] = v["description"]
                item["LoadingTime"] = v["createdTime"]
               	table.insert(tmp_tab, item)
            end
	    msg_ret["List"] = tmp_tab
        else
            utils.error_exit(200, json.encode({}))
        end
    else
    	utils.error_exit(200, json.encode({}))
    end 

    utils.error_exit(200, json.encode(msg_ret)) 

end
utils.error_exit(400, "unsupported request method")

