#!/usr/bin/env python3
# -*- coding: utf-8 -*-


from leda_python import leda
import logging
import threading
import json
import time
import os
import sys

_logger = logging.getLogger(__name__)

DRIVER_NAME = "demoDriver123"
PK = sys.argv[1]
DN = sys.argv[2]
asyncTimeOut = sys.argv[3]

fileName = 'testFile.txt'
FILE_NAME_PATH = "/tmp/" + fileName


class SyncMsg_Event(object):
    def __init__(self):
        self.event = threading.Event()
        self.clear()
        self.msg = None

    def wait(self, time_s):
        self.event.wait(time_s)

        return self.msg

    def set(self, msg):
        self.msg = msg
        self.event.set()

    def clear(self):
        self.msg = None
        self.event.clear()


class DeviceCallback(leda.BaseDeviceCallback):
    def __init__(self, userData=None):
        self.userData = userData

    def callService(self, name, input):
        pass

    def getprofile_cb(self):
        pass

    def getProperties(self, input):
        pass

    def setProperties(self, input):
        pass


class FileUploadResultCallback(leda.fileUploadResultCallback):
    def __init__(self, syncEvent, userData=None):
        self.userData = userData
        self.syncEvent = syncEvent

    def fileUploadResult(self, fileName, resultCode, message):
        msg = {}

        msgInfo = 'FileUploadResultNotice: \n'
        msgInfo += '\tfileName: %s\n' % (fileName)
        msgInfo += '\tresultCode: %s' % (resultCode)
        msgInfo += '\tUrl: %s' % (message)
        msg["code"] = resultCode
        msg["fileName"] = message

        _logger.info('%s', msgInfo)
        self.syncEvent.set(msg)


def test_report_uploadFile(driverObj, syncEvent):
    deviceObj = driverObj.deviceRegister(DN, PK, "", DeviceCallback())  # register && online

    while (1):
        driverObj.asyncAddFileUpload(fileList=[FILE_NAME_PATH, ])
        syncEvent.clear()
        retInfo = syncEvent.wait(int(asyncTimeOut))
        eventMsg = {}
        if (retInfo):
            if (retInfo['code'] == 0):
                eventMsg['state'] = 0
                eventMsg['fileName'] = fileName

            else:
                eventMsg['state'] = 1
                eventMsg['fileID'] = ''

        else:
            eventMsg['state'] = 1
            eventMsg['fileID'] = ''

        deviceObj.reportEvent('event_uploadResult', eventMsg)

        time.sleep(4)


if (__name__ == "__main__"):
    logging.basicConfig(level=logging.DEBUG,
                        format='%(asctime)s-%(threadName)-9s-%(filename)s[line:%(lineno)d]-%(levelname)s: %(message)s')

    _logger.debug("demo driver testing...")

    syncEvent = SyncMsg_Event()

    driverObj = leda.LedaModule()
    driverObj.moduleInit(DRIVER_NAME)
    driverObj.subFileUploadResult(FileUploadResultCallback(syncEvent))

    test_report_uploadFile(driverObj, syncEvent)

    while (1):
        time.sleep(1)
