#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import os

sys.path.append("../")
os.environ["FUNCTION_NAME"] = "driver_name"
try:
    import lethingaccesssdk
    import logging
    import json
    import mock
    import unittest
except ImportError:
    raise ImportError("import error")
fmt = "<%(filename)s:%(lineno)s> %(levelname)s - %(message)s"
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG, format=fmt)


class Demo_device(lethingaccesssdk.ThingCallback):
    def callService(self, name, input):
        pass

    def getProperties(self, input):
        pass

    def setProperties(self, input):
        pass


class TestThingAccessSDK(unittest.TestCase):
    def test_registerAndonline(self):
        config = 123  # wrong type
        client = lethingaccesssdk.ThingAccessClient(config)
        demo = Demo_device()
        result = None
        try:
            result = client.registerAndOnline(demo)
        except Exception as e:
            logging.error(e)
        self.assertEqual(result, None)

    def test_registerAndonline1(self):
        config = {"productKey": "a1ICUKj3Pyf"}  # miss parameter
        client = lethingaccesssdk.ThingAccessClient(config)
        demo = Demo_device()
        result = None
        try:
            result = client.registerAndOnline(demo)
        except Exception as e:
            logging.error(e)
        self.assertEqual(result, None)

    def test_registerAndonline2(self):
        config = {"productKey": 123456, "deviceName": "Hb9TYuRNqqw445xGYmu3"}  # wrong pk or dn
        client = lethingaccesssdk.ThingAccessClient(config)
        demo = Demo_device()
        result = None
        try:
            result = client.registerAndOnline(demo)
        except Exception as e:
            logging.error(e)
        self.assertEqual(result, None)

    def test_registerAndonline3(self):
        config = {"productKey": "a1ICUKj3Pyf", "deviceName": "Hb9TYuRNqqw445xGYmu3"}  # wrong callback
        client = lethingaccesssdk.ThingAccessClient(config)
        demo = None
        result = None
        try:
            result = client.registerAndOnline(demo)
        except Exception as e:
            logging.error(e)
        self.assertEqual(result, None)

    def test_reportEvent(self):
        config = {"productKey": "a1ICUKj3Pyf", "deviceName": "Hb9TYuRNqqw445xGYmu3"}
        client = lethingaccesssdk.ThingAccessClient(config)
        demo = Demo_device()
        result = None
        data = None
        try:
            result = client.registerAndOnline(demo)
            data = client.reportEvent("eventName", "abcd")
        except Exception as e:
            logging.error(e)
        client.offline()
        self.assertNotEqual(result, None)
        self.assertEqual(data, None)

    def test_reportEvent1(self):
        config = {"productKey": "a1ICUKj3Pyf", "deviceName": "Hb9TYuRNqqw445xGYmu3"}
        client = lethingaccesssdk.ThingAccessClient(config)
        demo = Demo_device()
        event = {"event_key": "value"}
        result = None
        data = None
        try:
            result = client.registerAndOnline(demo)
            data = client.reportEvent("eventName", event)
        except Exception as e:
            logging.error(e)
        client.offline()
        self.assertNotEqual(result, None)
        self.assertEqual(data, None)

    def test_reportProperty(self):
        config = {"productKey": "a1ICUKj3Pyf", "deviceName": "Hb9TYuRNqqw445xGYmu3"}
        client = lethingaccesssdk.ThingAccessClient(config)
        demo = Demo_device()
        client.registerAndOnline(demo)
        property = "property1"
        try:
            client.reportProperties(property)
        except Exception as e:
            logging.error(e)
        client.offline()

    def test_reportProperty1(self):
        config = {"productKey": "a1ICUKj3Pyf", "deviceName": "Hb9TYuRNqqw445xGYmu3"}
        client = lethingaccesssdk.ThingAccessClient(config)
        demo = Demo_device()
        property = {"property_key": "values"}
        try:
            client.registerAndOnline(demo)
            client.reportProperties(property)
        except Exception as e:
            logging.error(e)
        client.offline()

    def test_getConfig(self):
        config = {"productKey": "a1ICUKj3Pyf", "deviceName": "Hb9TYuRNqqw445xGYmu3"}
        ret_params = "{}"
        ret = mock.Mock(return_value=ret_params)

        client = lethingaccesssdk.ThingAccessClient(config)
        demo = Demo_device()
        try:
            client.registerAndOnline(demo)
        except Exception as e:
            logging.error(e)
        client.offline()


if __name__ == '__main__':
    unittest.main()
