#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from multiprocessing import Process
import json
import os 
import hashlib
import ctypes
from multiprocessing import Process
import time
import sys
import re
import math
import subprocess
import psutil
import csv_xlsx
import shutil
import zipfile
from jinja2 import Environment, FileSystemLoader 

ll = ctypes.cdll.LoadLibrary

g_test_pk='a1JLFi4YoHi'  
g_test_dn_prefix='Dev_'
g_output_dir='/tmp/'
g_benchmark_suite_dir='/linkedge/gateway/build/bin/test_tool/benchmark_suite/'
g_message_route='{"content":[{"source":{"context":{},"data":"#","type":"device"},"target":{"context":{"isLongLived":true},"data":"fcf9bbbe27958e4508beb6ee0c16b2ea00","type":"function"},"topicFilter":"all"}],"version":"1.0"}'
g_fc_test_output_file='/linkedge/run/benchmark_fc/csvdir.csv'
g_test_info_json=g_output_dir+'benchmark_info.json'
g_method1_test_json=g_output_dir+'benchmark_method1_test.json'

def zip_files(reports_dir):
    start_dir = reports_dir[0:reports_dir.rfind('/')]  # 要压缩的文件夹路径
    file_news = start_dir + '.zip'  # 压缩后文件夹的名字
    z = zipfile.ZipFile(file_news, 'w', zipfile.ZIP_DEFLATED)
    for dir_path, dir_names, file_names in os.walk(start_dir):
        f_path = dir_path.replace(start_dir, '')  # 这一句很重要，不replace的话，就从根目录开始复制
        f_path = f_path and f_path + os.sep or ''  # 实现当前文件夹以及包含的所有文件的压缩
        for filename in file_names:
            z.write(os.path.join(dir_path, filename), f_path + filename)
    z.close()
    return file_news

 
def generate_html(body, reports_dir):
    env = Environment(loader=FileSystemLoader('./'))
    template = env.get_template('template.html')     
    with open(reports_dir+"result.html",'w+', encoding='utf-8') as fout:   
        html_content = template.render(body=body)
        fout.write(html_content)

def generate_method1_html_report(body, reports_dir):
    env = Environment(loader=FileSystemLoader('./'))
    template = env.get_template('method1_template.html')     
    with open(reports_dir+"result.html",'w+', encoding='utf-8') as fout:   
        html_content = template.render(body=body)
        fout.write(html_content)    

def generate_device_trinity():
    global g_test_pk
    pk = g_test_pk  
    dn_prefix = 'Dev_'
    device_list = {
        "params": []
    }
    #dn_fmt = dn_prefix + '{0:04d}'
    #dn_fmt.format(1)
    for i in range(0,10000):
        dn = dn_prefix+str(i)
        m = hashlib.sha1()
        m.update(pk.encode('utf-8'))
        m.update(dn.encode('utf-8'))
        device_list['params'].append([pk,dn,m.hexdigest()])
    file_path=g_output_dir+'benchmark_trinity.json'
    with open(file_path,'w+') as f:
        f.write(json.dumps(device_list))
    return file_path

def import_device_trinity(file_path):
    global g_benchmark_suite_dir
    cmd=g_benchmark_suite_dir+'import-device-triple '+file_path
    os.system(cmd)
	
def kill_process(pid,signal):
    cmd='kill -'+str(signal)+' '+str(pid)
    print('kill process ',pid)
    os.system(cmd)
    while True:
        cmd='ps -eo pid | grep -w '+str(pid) + '| grep -v grep'
        r=os.popen(cmd)
        rt_text=r.read()
        r.close()
        if rt_text == None or rt_text == '':
            break
        else:
            print('Waitting process '+str(pid)+' exit, sleep 3s...')
            time.sleep(3)
            continue
    print('Process %d is killed'%pid)

def kill_remnant_process():
    cmd = "ps -ef | grep DriverPressureTestListner | grep -v grep | awk '{print $2}' | xargs -r kill -9"
    os.system(cmd)
    cmd = "ps -ef | grep fake-device | grep -v grep | awk '{print $2}' | xargs -r kill -9"
    os.system(cmd)

def deploy_fc():
    global g_benchmark_suite_dir
    cmd='unzip -qod /linkedge/run/benchmark_fc '+g_benchmark_suite_dir+'benchmark_fc.zip'
    #cmd = 'cp -r ./benchmark_fc /linkedge/run/'
    os.system(cmd)
    cmd='/linkedge/gateway/build/bin/fctl deploy '+g_benchmark_suite_dir+'benchmark_fc_deploy.conf'
    os.system(cmd)
    return

def reset_fc():
    cmd = '/linkedge/gateway/build/bin/fctl reset'
    os.system(cmd)
    return

def set_message_route_for_fc_test():
    global g_message_route
    cmd = '/linkedge/gateway/build/bin/lectl config set gw_messagerouter_localrule \''+g_message_route+'\''
    os.system(cmd)

def restart_linkedge():
    cmd = '/linkedge/gateway/build/script/iot_gateway_stop.sh all'
    os.system(cmd)
    cmd = '/linkedge/gateway/build/script/iot_gateway_start.sh'
    os.system(cmd)

def start_fake_device_with_multi_driver(driver_cnt,device_cnt,property_cnt_per_dev,report_cycle):
    global g_test_pk
    global g_test_dn_prefix
    global g_benchmark_suite_dir
    fake_device_output_file_list=[]
    driver_pid_list=[]    
    device_cnt_per_driver=device_cnt//driver_cnt
    for i in range(0,driver_cnt):
        fake_device_output_file=g_output_dir+'fake_device_output_%d.txt'%i
        fake_device_output_file_list.append(fake_device_output_file)
        start_num=device_cnt_per_driver*i
        if i == (driver_cnt-1):
            device_cnt_per_driver=device_cnt_per_driver+device_cnt%driver_cnt
        print('Starting driver %d with device %s%d - %s%d ...'%(i,g_test_dn_prefix,start_num,g_test_dn_prefix,start_num+device_cnt_per_driver-1))
        p=subprocess.Popen([g_benchmark_suite_dir+'fake-device',g_test_pk,str(start_num),str(device_cnt_per_driver),str(property_cnt_per_dev),str(report_cycle),g_test_dn_prefix,fake_device_output_file])
        driver_pid_list.append(p.pid)
    return fake_device_output_file_list,driver_pid_list

def stop_drivers(driver_pid_list):
    for pid in driver_pid_list:
        print('Stoping driver %d ...' % pid)
        kill_process(pid, 2)
    time.sleep(3)

def start_listener():
    global g_benchmark_suite_dir
    listener_output_file=g_output_dir+'listener_output_%d.csv'%int(time.time())
    print('Starting listener with output file ',listener_output_file)
    proc=subprocess.Popen([g_benchmark_suite_dir+'DriverPressureTestListner', listener_output_file])
    return listener_output_file, proc.pid

def stop_listener(listener_pid):
    print('Stoping listener %d ...' % listener_pid)
    kill_process(listener_pid, 2)
    time.sleep(3)

def get_system_info(system_info):
    with os.popen('cat /linkedge/gateway/build/VERSION') as process:
        le_version=process.read().strip()
    with os.popen('uname -a') as process:
        system_version=process.read().strip()
        if system_version=='':
            system_version='Unknown'
    with os.popen('cat /proc/cpuinfo | grep "model name" | cut -f2 -d: | uniq') as process:
        cpu_version=process.read().strip()
        if cpu_version=='':
            cpu_version='Unknown'
    with os.popen('cat /proc/cpuinfo | grep "physical id" | uniq | wc -l') as process:
        physcial_cpu=process.read().strip()
        if physcial_cpu=='' or physcial_cpu=='0':
            physcial_cpu='Unknown'
    with os.popen('cat /proc/cpuinfo | grep "core id" | uniq | wc -l') as process:
        cores_per_cpu=process.read().strip()
        if cores_per_cpu=='' or cores_per_cpu=='0':
            cores_per_cpu='Unknown'
    with os.popen('cat /proc/cpuinfo | grep "processor" | uniq | wc -l') as process:
        logical_cpu=process.read().strip()
        if logical_cpu=='' or logical_cpu=='0':
            with os.popen('lscpu | grep "CPU(s):" | cut -d: -f2'):
                logical_cpu=process.read().strip()
        if logical_cpu=='':
            logical_cpu='Unknown'
    with os.popen('cat /proc/meminfo | grep "MemTotal" | cut -f2 -d:') as process:
        memory=process.read().strip()
        if memory=='':
            memory='Unknown'
    with os.popen('arch') as process:
        cpu_arch=process.read().strip()
        if cpu_arch=='':
            cpu_arch='Unknown'
    system_info['CpuArch']=cpu_arch
    system_info['PhysicalCpu']=physcial_cpu
    system_info['CoresPerCpu']=cores_per_cpu
    system_info['LogicalCpu']=logical_cpu
    system_info['CpuVersioin']=cpu_version
    system_info['LinkIotEdgeVersion']=le_version
    system_info['Memory']=memory
    system_info['LinuxVersion']=system_version
    return 

def system_info_to_str(system_info): 
    system_info_str='\n\n系统信息：'+ \
            '\n    内存：'+system_info['Memory']+ \
            '\n    CPU架构：'+system_info['CpuArch']+ \
            '\n    物理CPU个数：'+system_info['PhysicalCpu']+ \
            '\n    每个CPU的核数：'+system_info['CoresPerCpu']+ \
            '\n    逻辑CPU个数:'+system_info['LogicalCpu']+ \
            '\n    CPU型号：'+system_info['CpuVersioin']+ \
            '\n    Linux版本：'+system_info['LinuxVersion']+ \
            '\n    LinkIoTEdge版本：'+system_info['LinkIotEdgeVersion']+ \
            '\n'
    return system_info_str

def get_system_load_avg():
    line=''
    with os.popen('cat /proc/loadavg') as process:
        line=process.read().strip()
    cut_position=line.rfind(' ')
    sub_str=line[0:cut_position]
    cut_position=sub_str.rfind(' ')    
    load_avg=line[0:cut_position]
    return load_avg

def parse_fake_device_output_file(fake_device_output_file_list):
    sent_reports=0
    for file_name in fake_device_output_file_list:
        with open(file_name) as f:
            line=f.readline()
        line_array=re.split(',',line)
        sent_reports+=int(line_array[0][len('reports: '):])
    return sent_reports

def parse_listener_output_file(file_path, outputs):
    with open(file_path) as f:
        first_line=f.readline()
    split_array=re.split(',',first_line)
    duration_avg=split_array[0][len('AVG: '):split_array[0].rfind(' ')]
    duration_sd=split_array[1][len('SD: '):split_array[1].rfind(' ')]
    duration_max=split_array[2][len('Max: '):split_array[2].rfind(' ')]
    duration_min=split_array[3][len('Min: '):split_array[3].rfind(' ')]
    tmp=split_array[4][len('count: '):split_array[4].rfind(' ')]
    received_reports=tmp.split()[0]
    outputs['DurationAvg']=float(duration_avg)
    outputs['MaxDuration']=float(duration_max)
    outputs['MinDuration']=float(duration_min)
    outputs['DurationSD']=float(duration_sd)
    outputs['ReciveMsgTotal']=int(received_reports)
    outputs['ListenerOutputCsv']=file_path
    return

def parse_fc_test_output_file(fc_test_output_file,outputs):
    with open(fc_test_output_file) as f:
        i=0
        duration_min=99999
        duration_max=0
        duration_avg=0
        duration_sd=0
        duration_total=0
        duration_powed=0
        for line in f:
            i+=1
            if i==1:
                continue
            duration=float(line)
            if duration>duration_max:
                duration_max=duration
            if duration<duration_min:
                duration_min=duration
            duration_total+=duration
            duration_powed+=duration*duration
        duration_avg=duration_total/i
        duration_sd=math.sqrt(duration_powed/i-duration_avg*duration_avg)
    outputs['ReciveMsgTotal']=i-1
    outputs['DurationAvg']=round(duration_avg/1000, 3)
    outputs['MaxDuration']=round(duration_max/1000, 3)
    outputs['MinDuration']=round(duration_min/1000, 3)
    outputs['DurationSD']=round(duration_sd/1000,3)
    outputs['FcOutputCsv']=fc_test_output_file
    return

def test_type1_info_to_str(test_type1_info):
    inputs=test_type1_info['Input']
    outputs=test_type1_info['Output']
    test_type1_info_str='\n常规设备接入测试：'+ \
                    '\n    测试参数：'+ \
                    '\n        驱动进程数：'+str(inputs['DriverCnt'])+ \
                    '\n        设备数：'+str(inputs['DeviceCnt'])+ \
                    '\n        点数/设备：'+str(inputs['PropertyCntPerDev'])+ \
                    '\n        上报周期（毫秒）：'+str(inputs['ReportCycle'])+ \
                    '\n        测试时间（秒）：'+str(inputs['TestTime'])+ \
                    '\n    测试结果：'+\
                    '\n        上报消息数：'+str(outputs['ReportMsgTotal'])+ \
                    '\n        接收消息数：'+str(outputs['ReciveMsgTotal'])+ \
                    '\n        发送接收耗时：'+ \
                    '\n            平均值(毫秒)：'+str(outputs['DurationAvg'])+ \
                    '\n            最大值(毫秒)：'+str(outputs['MaxDuration'])+ \
                    '\n            最小值(毫秒)：'+str(outputs['MinDuration'])+ \
                    '\n            标准差(毫秒)：'+str(outputs['DurationSD'])+ \
                    '\n    系统负载(load average)：'+outputs['LoadAverage']+ \
                    '\n'
    return test_type1_info_str

def test_type2_info_to_str(test_type2_info):
    inputs=test_type2_info['Input']
    outputs=test_type2_info['Output']
    test_type2_info_str='\n最大设备数测试：'+ \
                    '\n    测试参数：'+ \
                    '\n        驱动进程数：'+str(inputs['DriverCnt'])+ \
                    '\n        设备数基数：'+str(inputs['DeviceCntBase'])+ \
                    '\n        设备数歩进：'+str(inputs['Step'])+ \
                    '\n        点数/设备：'+str(inputs['PropertyCntPerDev'])+ \
                    '\n        上报周期（毫秒）：'+str(inputs['ReportCycle'])+ \
                    '\n        发送接收耗时平均值限制（毫秒）：'+str(inputs['DurationAvgLimit'])+ \
                    '\n        测试周期（秒）：'+str(inputs['TestCycle'])+ \
                    '\n        最大测试轮数：'+str(inputs['MaxTestTimes'])+ \
                    '\n    测试结果：'+\
                    '\n        最大可达接入设备数：'+str(outputs['MaxDeviceCnt'])+ \
                    '\n        上报消息数：'+str(outputs['ReportMsgTotal'])+ \
                    '\n        接收消息数：'+str(outputs['ReciveMsgTotal'])+ \
                    '\n        发送接收耗时：'+ \
                    '\n            平均值(毫秒)：'+str(outputs['DurationAvg'])+ \
                    '\n            最大值(毫秒)：'+str(outputs['MaxDuration'])+ \
                    '\n            最小值(毫秒)：'+str(outputs['MinDuration'])+ \
                    '\n            标准差(毫秒)：'+str(outputs['DurationSD'])+ \
                    '\n    系统负载(load average)：'+outputs['LoadAverage']+ \
                    '\n'
    return test_type2_info_str

def test_type3_info_to_str(test_type3_info):
    inputs=test_type3_info['Input']
    outputs=test_type3_info['Output']
    test_type3_info_str='\n最大属性数测试：'+ \
                    '\n    测试参数：'+ \
                    '\n        驱动进程数：'+str(inputs['DriverCnt'])+ \
                    '\n        设备数：'+str(inputs['DeviceCnt'])+ \
                    '\n        点数基数/设备：'+str(inputs['PropertyCntPerDevBase'])+ \
                    '\n        点数歩进/设备：'+str(inputs['Step'])+ \
                    '\n        上报周期（毫秒）：'+str(inputs['ReportCycle'])+ \
                    '\n        发送接收耗时平均值限制（毫秒）：'+str(inputs['DurationAvgLimit'])+ \
                    '\n        测试周期（秒）：'+str(inputs['TestCycle'])+ \
                    '\n        最大测试轮数：'+str(inputs['MaxTestTimes'])+ \
                    '\n    测试结果：'+\
                    '\n        最大可达接入属性点总数：'+str(outputs['MaxPropertyCntTotal'])+ \
                    '\n        上报消息数：'+str(outputs['ReportMsgTotal'])+ \
                    '\n        接收消息数：'+str(outputs['ReciveMsgTotal'])+ \
                    '\n        发送接收耗时：'+ \
                    '\n            平均值(毫秒)：'+str(outputs['DurationAvg'])+ \
                    '\n            最大值(毫秒)：'+str(outputs['MaxDuration'])+ \
                    '\n            最小值(毫秒)：'+str(outputs['MinDuration'])+ \
                    '\n            标准差(毫秒)：'+str(outputs['DurationSD'])+ \
                    '\n    系统负载(load average)：'+outputs['LoadAverage']+ \
                    '\n'
    return test_type3_info_str

def test_type4_info_to_str(test_type4_info):
    inputs=test_type4_info['Input']
    outputs=test_type4_info['Output']
    test_type4_info_str='\n设备-函数计算测试：'+ \
                    '\n    测试参数：'+ \
                    '\n        驱动进程数：'+str(inputs['DriverCnt'])+ \
                    '\n        设备数：'+str(inputs['DeviceCnt'])+ \
                    '\n        点数/设备：'+str(inputs['PropertyCntPerDev'])+ \
                    '\n        上报周期（毫秒）：'+str(inputs['ReportCycle'])+ \
                    '\n        测试时间（秒）：'+str(inputs['TestTime'])+ \
                    '\n    测试结果：'+\
                    '\n        上报消息数：'+str(outputs['ReportMsgTotal'])+ \
                    '\n        接收消息数：'+str(outputs['ReciveMsgTotal'])+ \
                    '\n        发送接收耗时：'+ \
                    '\n            平均值(毫秒)：'+str(outputs['DurationAvg'])+ \
                    '\n            最大值(毫秒)：'+str(outputs['MaxDuration'])+ \
                    '\n            最小值(毫秒)：'+str(outputs['MinDuration'])+ \
                    '\n            标准差(毫秒)：'+str(outputs['DurationSD'])+ \
                    '\n    系统负载(load average)：'+outputs['LoadAverage']+ \
                    '\n'
    return test_type4_info_str

def print_test_result(params):
    all_info=''
    all_info+=system_info_to_str(params['System'])
    if 'TestType1' in params.keys():
        all_info+=test_type1_info_to_str(params['TestType1'])
    if 'TestType2' in params.keys():
        all_info+=test_type2_info_to_str(params['TestType2'])
    if 'TestType3' in params.keys():
        all_info+=test_type3_info_to_str(params['TestType3'])
    if 'TestType4' in params.keys():
        all_info+=test_type4_info_to_str(params['TestType4'])
    print(all_info)
    return

def print_method1_test_result(params):
    all_info=''
    all_info+=system_info_to_str(params['System'])
    MaxDeviceCnt=params['MaxDeviceCnt']
    MaxPropertyCntTotal=params['MaxPropertyCntTotal']
    LongTimeTest=params['LongTimeTest']
    MaxDeviceCntInfo='\n最大设备数测试：'+ \
                    '\n    测试参数：'+ \
                    '\n        驱动进程数：'+ str(MaxDeviceCnt['DriverCnt']) + \
                    '\n        点数/设备：' + str(MaxDeviceCnt['PropertyCntPerDev']) + \
                    '\n        上报周期（毫秒）：'+ str(MaxDeviceCnt['ReportCycle']) + \
                    '\n    测试结果：'+ \
                    '\n        最大设备数：' + str(MaxDeviceCnt['DeviceCnt']) + \
                    '\n        上报消息数：' + str(MaxDeviceCnt['ReportMsgTotal']) + \
                    '\n        接收消息数：' + str(MaxDeviceCnt['ReciveMsgTotal']) + \
                    '\n        延时平均值(毫秒)：' + str(MaxDeviceCnt['DurationAvg']) + \
                    '\n        延时最大值(毫秒)：' + str(MaxDeviceCnt['MaxDuration']) + \
                    '\n        延时最小值(毫秒)：' + str(MaxDeviceCnt['MinDuration']) + \
                    '\n        延时标准差(毫秒)：' + str(MaxDeviceCnt['DurationSD']) + \
                    '\n        系统负载：' + MaxDeviceCnt['LoadAverage']  + \
                    '\n        CPU使用率：' + str(MaxDeviceCnt['CpuUsage'])
    all_info+=MaxDeviceCntInfo
    MaxPropertyCntInfo='\n最大属性点数测试：'+ \
                    '\n    测试参数：'+ \
                    '\n        驱动进程数：'+str(MaxPropertyCntTotal['DriverCnt']) + \
                    '\n        设备数：'+str(MaxPropertyCntTotal['DeviceCnt']) + \
                    '\n        上报周期：'+str(MaxPropertyCntTotal['ReportCycle']) + \
                    '\n    测试结果：'+ \
                    '\n        最大点数/设备：'+str(MaxPropertyCntTotal['PropertyCntPerDev']) + \
                    '\n        最大点数总数：'+str(MaxPropertyCntTotal['MaxPropertyCntTotal']) + \
                    '\n        上报消息数：' + str(MaxPropertyCntTotal['ReportMsgTotal']) + \
                    '\n        接收消息数：' + str(MaxPropertyCntTotal['ReciveMsgTotal']) + \
                    '\n        延时平均值(毫秒)：' + str(MaxPropertyCntTotal['DurationAvg']) + \
                    '\n        延时最大值(毫秒)：' + str(MaxPropertyCntTotal['MaxDuration']) + \
                    '\n        延时最小值(毫秒)：' + str(MaxPropertyCntTotal['MinDuration']) + \
                    '\n        延时标准差(毫秒)：' + str(MaxPropertyCntTotal['DurationSD']) + \
                    '\n        系统负载：' + MaxPropertyCntTotal['LoadAverage']  + \
                    '\n        CPU使用率：' + str(MaxPropertyCntTotal['CpuUsage'])
    all_info+=MaxPropertyCntInfo
    LongTimeTestInfo='\n长时间挂测：' + \
                    '\n    测试参数：' + \
                    '\n        驱动进程数：' + str(LongTimeTest['DriverCnt']) + \
                    '\n        设备数：' + str(LongTimeTest['DeviceCnt']) + \
                    '\n        点数/设备：' + str(LongTimeTest['PropertyCntPerDev']) + \
                    '\n        上报周期：' + str(LongTimeTest['ReportCycle']) + \
                    '\n    测试结果：' \
                    '\n        上报消息数：' + str(LongTimeTest['ReportMsgTotal']) + \
                    '\n        接收消息数：' + str(LongTimeTest['ReciveMsgTotal']) + \
                    '\n        延时平均值(毫秒)：' + str(LongTimeTest['DurationAvg']) + \
                    '\n        延时最大值(毫秒)：' + str(LongTimeTest['MaxDuration']) + \
                    '\n        延时最小值(毫秒)：' + str(LongTimeTest['MinDuration']) + \
                    '\n        延时标准差(毫秒)：' + str(LongTimeTest['DurationSD']) + \
                    '\n        系统负载：' + LongTimeTest['LoadAverage']  + \
                    '\n        CPU使用率：' + str(LongTimeTest['CpuUsage'])
    all_info+=LongTimeTestInfo
    print(all_info)

def get_system_load_avg():
    line=''
    with os.popen('cat /proc/loadavg') as process:
        line=process.read().strip()
    cut_position=line.rfind(' ')
    sub_str=line[0:cut_position]
    cut_position=sub_str.rfind(' ')    
    load_avg=line[0:cut_position]
    return load_avg  

def test_for_typex(params, test_type):
    global g_test_pk
    global g_test_dn_prefix
    global g_output_dir
    global g_results_dir
    global g_listener_output_file
    global g_fc_zip_path
    global g_fc_config_path
    global g_fc_test_output_file
    fake_device_output_file_list=[]
    driver_pid_list=[]
    file_path = generate_device_trinity()
    import_device_trinity(file_path)
    if test_type == 1:
        inputs=params['TestType1']['Input']
        outputs=params['TestType1']['Output']
        listener_output_file,listener_pid = start_listener()
        fake_device_output_file_list,driver_pid_list=start_fake_device_with_multi_driver(inputs['DriverCnt'],inputs['DeviceCnt'],inputs['PropertyCntPerDev'],inputs['ReportCycle'])
        time.sleep(inputs['TestTime'])
        outputs['LoadAverage']=get_system_load_avg()
        outputs['CpuUsage']=get_vmstat_cpu_usage()
        stop_drivers(driver_pid_list)
        stop_listener(listener_pid)
        parse_listener_output_file(listener_output_file, outputs)
        outputs['ReportMsgTotal']=parse_fake_device_output_file(fake_device_output_file_list)
    elif test_type == 2:
        inputs=params['TestType2']['Input']
        max_test_times=inputs['MaxTestTimes']
        device_cnt=inputs['DeviceCntBase']
        outputs=params['TestType2']['Output']
        outputs['DurationAvg']=0
        i=0
        while outputs['DurationAvg'] <= inputs['DurationAvgLimit']:
            outputs_record=outputs.copy()
            max_device_cnt=device_cnt
            if outputs['DurationAvg'] == inputs['DurationAvgLimit']:
                break            
            if max_test_times<=0:
                break
            device_cnt+=inputs['Step']
            if device_cnt>9999:
                break
            listener_output_file,listener_pid = start_listener()
            fake_device_output_file_list,driver_pid_list=start_fake_device_with_multi_driver(inputs['DriverCnt'],device_cnt,inputs['PropertyCntPerDev'],inputs['ReportCycle'])
            time.sleep(inputs['TestCycle'])
            outputs['LoadAverage']=get_system_load_avg()
            outputs['CpuUsage']=get_vmstat_cpu_usage()
            stop_drivers(driver_pid_list)
            stop_listener(listener_pid)
            parse_listener_output_file(listener_output_file, outputs)
            outputs['ReportMsgTotal']=parse_fake_device_output_file(fake_device_output_file_list)
            max_test_times-=1
            i+=1
        if outputs['DurationAvg'] > inputs['DurationAvgLimit'] and i==1 :
            print("The duration average is lager than the limit for first test, You should retype parameters again to get accurate result.")
        else:            
            params['TestType2']['Output']=outputs_record.copy()
        params['TestType2']['Output']['MaxDeviceCnt']=max_device_cnt
    elif test_type == 3:
        inputs=params['TestType3']['Input']
        max_test_times=inputs['MaxTestTimes']
        property_cnt_per_dev=inputs['PropertyCntPerDevBase']
        outputs=params['TestType3']['Output']
        outputs['DurationAvg']=0
        i=0
        while outputs['DurationAvg'] <= inputs['DurationAvgLimit']:
            outputs_record=outputs.copy()
            max_property_cnt_per_dev=property_cnt_per_dev
            if outputs['DurationAvg'] == inputs['DurationAvgLimit']:
                break            
            if max_test_times<=0:
                break
            property_cnt_per_dev+=inputs['Step']
            if property_cnt_per_dev>200:
                break
            listener_output_file,listener_pid = start_listener()
            fake_device_output_file_list,driver_pid_list=start_fake_device_with_multi_driver(inputs['DriverCnt'],inputs['DeviceCnt'],property_cnt_per_dev,inputs['ReportCycle'])
            time.sleep(inputs['TestCycle'])
            outputs['LoadAverage']=get_system_load_avg()
            stop_drivers(driver_pid_list)
            stop_listener(listener_pid)
            parse_listener_output_file(listener_output_file, outputs)
            outputs['ReportMsgTotal']=parse_fake_device_output_file(fake_device_output_file_list)
            max_test_times-=1
            i+=1
        if outputs['DurationAvg'] > inputs['DurationAvgLimit'] and i==1 :
            print("The duration average is lager than the limit for first test, You should retype parameters again to get accurate result.")
        else:
            params['TestType3']['Output']=outputs_record.copy()
        params['TestType3']['Output']['MaxPropertyCntTotal']=max_property_cnt_per_dev*inputs['DeviceCnt']
    elif test_type == 4:
        deploy_fc()
        set_message_route_for_fc_test()
        restart_linkedge()
        inputs=params['TestType4']['Input']
        outputs=params['TestType4']['Output']
        fake_device_output_file_list,driver_pid_list=start_fake_device_with_multi_driver(inputs['DriverCnt'],inputs['DeviceCnt'],inputs['PropertyCntPerDev'],inputs['ReportCycle'])
        time.sleep(inputs['TestTime'])
        outputs['LoadAverage']=get_system_load_avg()
        stop_drivers(driver_pid_list)
        print('Waiting fc handle timeout for 120s...')
        time.sleep(120)
        reset_fc()
        benchmark_fc_file=g_output_dir+'benchmark_fc_%d.csv'%int(time.time())
        shutil.copy(g_fc_test_output_file, benchmark_fc_file)
        outputs['ReportMsgTotal']=parse_fake_device_output_file(fake_device_output_file_list)
        parse_fc_test_output_file(benchmark_fc_file,outputs)
    return

def generate_report_xlsx(params, test_type, output_dir):
    if test_type==1:
        csv_file=params['TestType1']['Output']['ListenerOutputCsv']
    elif test_type==2:
         csv_file=params['TestType2']['Output']['ListenerOutputCsv']
    elif test_type==3:
         csv_file=params['TestType3']['Output']['ListenerOutputCsv']
    elif test_type==4:
         csv_file=params['TestType4']['Output']['FcOutputCsv']
    else:
        sys.exit(1)
    xlsx_file=csv_xlsx.csv_to_xlsx(csv_file, output_dir)
    if test_type==1:
        params['TestType1']['Output']['Xlsx']=xlsx_file
    elif test_type==2:
        params['TestType2']['Output']['Xlsx']=xlsx_file
    elif test_type==3:
        params['TestType3']['Output']['Xlsx']=xlsx_file
    elif test_type==4:
        params['TestType4']['Output']['Xlsx']=xlsx_file
    else:
        sys.exit(1)    
    return xlsx_file

def copy_xlsx_files(params, reports_dir):
    if 'TestType1' in params.keys():
        shutil.copy(params['TestType1']['Output']['Xlsx'], reports_dir)
    if 'TestType2' in params.keys():
        shutil.copy(params['TestType2']['Output']['Xlsx'], reports_dir)
    if 'TestType3' in params.keys():
        shutil.copy(params['TestType3']['Output']['Xlsx'], reports_dir)
    if 'TestType4' in params.keys():
        shutil.copy(params['TestType4']['Output']['Xlsx'], reports_dir)

def test_with_method_2():
    params={'System':{}}
    if os.path.exists(g_test_info_json):
        with open(g_test_info_json, 'r') as f:
            content=f.read()
            if content=='':
                get_system_info(params['System'])
            else:
                params=json.loads(content)
    else:
        get_system_info(params['System'])    
    test_type_str=input('测试类型：\n    1、(常规设备接入测试):该测试在您待接入设备数、每个设备的属性点数、驱动进程个数以及采集（上报）频率确定的情况下，对您所测的网关性能进行测试评估。\n    2、(最大设备数测试):该测试在您每个设备属性点数、驱动进程个数以及采集频率（上报）确定的情况下，对您所测的网关所能支持的最大设备数目进行评估。\n    3、(最大属性数测试):该测试在您设备数、驱动进程个数以及采集（上报）频率确定的情况下，对您所测的网关所能支持的最大设备总属性点数进行评估。\n    4、(设备-函数计算测试):该测试在您设备数，每个设备属性点数、驱动进程个数以及采集（上报）频率确定的情况下，对您所测的网关，从设备到函数计算的消息承载能力进行评估。\n请选择（1 or 2 or 3 or 4）：')
    test_type=int(test_type_str)
    if test_type==1:
        driver_cnt=int(input('驱动进程数：'))
        device_cnt=int(input('设备数（1~9999）：'))
        if device_cnt < 1 or device_cnt > 9999:
            print('无效输入！')
            sys.exit(1)
        property_cnt_per_dev=int(input('点数/设备（2~200）：'))
        if property_cnt_per_dev < 2 or property_cnt_per_dev > 200:
            print('无效输入！')
            sys.exit(1)
        report_cycle=int(input('上报周期（毫秒）：'))
        test_time=int(input('测试时间（秒，>=60）：'))
        if test_time<60:
            print('无效输入!')
            sys.exit(1)
        params['TestType1']={'Input':{},'Output':{}}
        params['TestType1']['Input']={
            'DriverCnt':driver_cnt,
            'DeviceCnt':device_cnt,
            'PropertyCntPerDev':property_cnt_per_dev,
            'ReportCycle':report_cycle,
            'TestTime':test_time,
        }
    elif test_type==2:
        driver_cnt=int(input('驱动进程数：'))
        device_cnt_base=int(input('设备数基数（1~9999）：'))
        if device_cnt_base < 1 or device_cnt_base > 9999:
            print('无效输入！')
            sys.exit(1)
        step=int(input('设备数歩进：'))
        property_cnt_per_dev=int(input('点数/设备（2~200）：'))
        if property_cnt_per_dev < 2 or property_cnt_per_dev > 200:
            print('无效输入！')
            sys.exit(1)
        report_cycle=int(input('上报周期（毫秒）：'))
        duration_avg_limit=int(input('发送接收耗时平均值限制（毫秒）:'))
        test_cycle=int(input('测试周期（秒，>=60）：'))
        if test_cycle<60:
            print('无效输入!')
            sys.exit(1)
        max_test_times=int(input('最大测试轮数(>=1)：'))
        params['TestType2']={'Input':{},'Output':{}}
        params['TestType2']['Input']={
            'DriverCnt':driver_cnt,
            'DeviceCntBase':device_cnt_base,
            'Step':step,
            'PropertyCntPerDev':property_cnt_per_dev,
            'ReportCycle':report_cycle,
            'DurationAvgLimit':duration_avg_limit,
            'TestCycle':test_cycle,
            'MaxTestTimes':max_test_times
        }        
    elif test_type==3:
        driver_cnt=int(input('驱动进程数：'))
        device_cnt=int(input('设备数（1~9999）：'))
        if device_cnt < 1 or device_cnt > 9999:
            print('无效输入！')
            sys.exit(1)
        property_cnt_per_dev_base=int(input('点数基数/设备（2~200）：'))
        if property_cnt_per_dev_base < 2 or property_cnt_per_dev_base > 200:
            print('无效输入！')
            sys.exit(1)
        step=int(input('点数歩进/设备：'))
        report_cycle=int(input('上报周期（毫秒）：'))
        duration_avg_limit=int(input('发送接收耗时平均值限制（毫秒）:'))
        test_cycle=int(input('测试周期（秒，>=60）：'))
        if test_cycle<60:
            print('无效输入!')
            sys.exit(1)
        max_test_times=int(input('最大测试轮数(>=1)：'))
        params['TestType3']={'Input':{},'Output':{}}
        params['TestType3']['Input']={
            'DriverCnt':driver_cnt,
            'DeviceCnt':device_cnt,
            'PropertyCntPerDevBase':property_cnt_per_dev_base,
            'Step':step,            
            'ReportCycle':report_cycle,
            'DurationAvgLimit':duration_avg_limit,
            'TestCycle':test_cycle,
            'MaxTestTimes':max_test_times
        }        
    elif test_type==4:
        driver_cnt=int(input('驱动进程数：'))
        device_cnt=int(input('设备数（1~9999）：'))
        if device_cnt < 1 or device_cnt > 9999:
            print('无效输入！')
            sys.exit(1)
        property_cnt_per_dev=int(input('点数/设备（2~200）：'))
        if property_cnt_per_dev < 2 or property_cnt_per_dev > 200:
            print('无效输入！')
            sys.exit(1)
        report_cycle=int(input('上报周期（毫秒）：'))
        test_time=int(input('测试时间（秒，>=60）：'))
        if test_time<60:
            print('无效输入!')
            sys.exit(1)
        params['TestType4']={'Input':{},'Output':{}}
        params['TestType4']['Input']={
            'DriverCnt':driver_cnt,
            'DeviceCnt':device_cnt,
            'PropertyCntPerDev':property_cnt_per_dev,
            'ReportCycle':report_cycle,
            'TestTime':test_time,
        }
    else:
        pass
    kill_remnant_process()
    restart_linkedge()
    test_for_typex(params, test_type)
    reports_dir=g_output_dir+'reports_%s/'%str(int(time.time()))
    generate_report_xlsx(params, test_type, g_output_dir)
    os.mkdir(reports_dir)
    generate_html(params, reports_dir)
    copy_xlsx_files(params, reports_dir)
    reports_zip_file=zip_files(reports_dir)
    with open(g_test_info_json, 'w+', encoding='utf-8') as f:
        json.dump(params, f)
    print_test_result(params)
    print("测试报告文件：%s\n"%reports_zip_file)


def test_with_method_1():
    if os.path.exists(g_method1_test_json):
        with open(g_method1_test_json, 'r') as f:
            content=f.read()
            if content != '':
                params=json.loads(content)
            else:
                params={
                            'System':{},
                            'MaxDeviceCnt':{
                                'DriverCnt':1,
                                'ReportCycle':1000,
                                'PropertyCntPerDev':10
                            },
                            'MaxPropertyCntTotal':{
                                'DriverCnt':1,
                                'ReportCycle':1000
                            },
                            'LongTimeTest': {
                                'DriverCnt':1,
                                'ReportCycle':1000
                            }
                        }
                get_system_info(params['System'])                                 
    upper_limit=9999
    lower_limit=1
    reports_dir=g_output_dir+'reports_%s/'%str(int(time.time()))
    os.mkdir(reports_dir)
    file_path = generate_device_trinity()
    import_device_trinity(file_path)
    kill_remnant_process()
    restart_linkedge()
    MaxDeviceCnt=params['MaxDeviceCnt']
    while True:
        MaxDeviceCnt['DeviceCnt']=(upper_limit+lower_limit)//2
        listener_output_file,listener_pid = start_listener()
        fake_device_output_file_list,driver_pid_list = start_fake_device_with_multi_driver(MaxDeviceCnt['DriverCnt'], MaxDeviceCnt['DeviceCnt'], MaxDeviceCnt['PropertyCntPerDev'], MaxDeviceCnt['ReportCycle'])
        time.sleep(1200)
        MaxDeviceCnt['LoadAverage']=get_system_load_avg()
        MaxDeviceCnt['CpuUsage'] = get_vmstat_cpu_usage()
        stop_drivers(driver_pid_list)
        stop_listener(listener_pid)
        parse_listener_output_file(listener_output_file, MaxDeviceCnt)
        MaxDeviceCnt['ReportMsgTotal']=parse_fake_device_output_file(fake_device_output_file_list) 
        if upper_limit-lower_limit<=10:
                break
        if MaxDeviceCnt['CpuUsage'] > 80:
            upper_limit = MaxDeviceCnt['DeviceCnt']
        elif 80 - MaxDeviceCnt['CpuUsage'] > 5:
            lower_limit = MaxDeviceCnt['DeviceCnt']
        else:
            break
        time.sleep(10)
    MaxDeviceCnt['Xlsx']=csv_xlsx.csv_to_xlsx(MaxDeviceCnt['ListenerOutputCsv'], reports_dir)

    upper_limit=200
    lower_limit=2
    MaxPropertyCntTotal=params['MaxPropertyCntTotal']
    MaxPropertyCntTotal['DeviceCnt']=MaxDeviceCnt['DeviceCnt']//2
    kill_remnant_process()
    restart_linkedge()
    while True:
        MaxPropertyCntTotal['PropertyCntPerDev']=(upper_limit+lower_limit)//2
        MaxPropertyCntTotal['MaxPropertyCntTotal']=MaxPropertyCntTotal['PropertyCntPerDev']*MaxPropertyCntTotal['DeviceCnt']
        listener_output_file,listener_pid = start_listener()
        fake_device_output_file_list,driver_pid_list = start_fake_device_with_multi_driver(MaxPropertyCntTotal['DriverCnt'], MaxPropertyCntTotal['DeviceCnt'], MaxPropertyCntTotal['PropertyCntPerDev'], MaxPropertyCntTotal['ReportCycle'])
        time.sleep(1200)
        MaxPropertyCntTotal['LoadAverage']=get_system_load_avg()
        MaxPropertyCntTotal['CpuUsage'] = get_vmstat_cpu_usage()
        stop_drivers(driver_pid_list)
        stop_listener(listener_pid)
        parse_listener_output_file(listener_output_file, MaxPropertyCntTotal)
        MaxPropertyCntTotal['ReportMsgTotal']=parse_fake_device_output_file(fake_device_output_file_list) 
        if upper_limit-lower_limit<=10:
                break
        if MaxDeviceCnt['CpuUsage'] > 80:
            upper_limit = MaxPropertyCntTotal['PropertyCntPerDev']
        elif 80 - MaxDeviceCnt['CpuUsage'] > 5:
            lower_limit = MaxPropertyCntTotal['PropertyCntPerDev']
        else:
            break
        time.sleep(10)
    MaxPropertyCntTotal['Xlsx']=csv_xlsx.csv_to_xlsx(MaxPropertyCntTotal['ListenerOutputCsv'], reports_dir)

    kill_remnant_process()
    restart_linkedge()
    LongTimeTest=params['LongTimeTest']
    LongTimeTest['PropertyCntPerDev']=MaxPropertyCntTotal['PropertyCntPerDev']//2
    LongTimeTest['DeviceCnt']=MaxDeviceCnt['DeviceCnt']//2
    listener_output_file,listener_pid = start_listener()
    fake_device_output_file_list,driver_pid_list = start_fake_device_with_multi_driver(LongTimeTest['DriverCnt'], LongTimeTest['DeviceCnt'], LongTimeTest['PropertyCntPerDev'], LongTimeTest['ReportCycle'])
    time.sleep(6*3600)
    LongTimeTest['LoadAverage']=get_system_load_avg()
    LongTimeTest['CpuUsage'] = get_vmstat_cpu_usage()
    stop_drivers(driver_pid_list)
    stop_listener(listener_pid)
    parse_listener_output_file(listener_output_file, LongTimeTest)
    LongTimeTest['ReportMsgTotal']=parse_fake_device_output_file(fake_device_output_file_list)     
    LongTimeTest['Xlsx']=csv_xlsx.csv_to_xlsx(LongTimeTest['ListenerOutputCsv'], reports_dir)
    
    generate_method1_html_report(params, reports_dir)
    reports_zip_file=zip_files(reports_dir)
    with open(g_method1_test_json, 'w+', encoding='utf-8') as f:
        json.dump(params, f)
    print_method1_test_result(params)
    print("测试报告文件：%s\n"%reports_zip_file)

def get_vmstat_cpu_usage():
    no_idle_list=[]
    total=0
    i=6
    while i>0:
        cpu_usage_str=''
        with os.popen('vmstat 1 21 | sed -n "4,23p"|awk -F " " \'{for (i=NF-4;i<=NF;i++)printf("%s ", $i);print ""}\'') as process:
            cpu_usage_str=process.read().strip()
        tmp=cpu_usage_str.split(' ')
        cpu_usage=[]
        for n in tmp: 
            cpu_usage.append(int(n))
        no_idle=cpu_usage[0]+cpu_usage[1]+cpu_usage[3]+cpu_usage[4]
        no_idle_list.append(no_idle)
        i-=1
    for value in no_idle_list:
        total+=value
    no_idle_avg=total/20
    return no_idle_avg 


if __name__ == "__main__":
    if len(sys.argv)  > 1:
        argv1=sys.argv[1]
        print('argv1=', argv1)
        if argv1 == '1':
            test_with_method_1()
        elif argv1 == '2':
            test_with_method_2()
        else:
            sys.exit(1)
    else:                        
        test_method=input('测试方式：\n    1、全自动化测试(时间长，需要1天时间)\n    2、半自动化测试（测试参数可配）\n请选择（1 or 2）：')
        if test_method == '1':
            test_with_method_1()
        elif test_method == '2':
            test_with_method_2()
        else:
            print('无效输入')
            sys.exit(1)
